
import numpy as np
import pandas as pd
import xarray as xr
import scipy


def read_S5P_NO2_matlab_file(filename):
    """
    Reads TROPOMI NO2 data from matlab files created in CoCO2 project.
    """
    F = scipy.io.loadmat(filename)

    time = np.asarray([pd.to_datetime(time)
        for time in F['time_utc'][:,0]])[:,0].astype('datetime64[ns]')

    shape = F['nitrogendioxide_tropospheric_column'].T.shape

    data = xr.Dataset(
      data_vars = dict(
          NO2 = (["nobs", "nrows"], F['nitrogendioxide_tropospheric_column'].T),
          NO2_std=(["nobs", "nrows"],
                   F['nitrogendioxide_tropospheric_column_precision'].T),
          latc=(["nobs", "nrows", "corner"], F['latitude_bounds'].T),
          lonc=(["nobs", "nrows", "corner"], F['longitude_bounds'].T),
          time_utc=(["nobs"], time ),
          psurf=(["nobs", "nrows"], F['surface_pressure'].T),
          clouds=(['nobs', 'nrows'], np.full(shape, 0.0))
      ),
      coords=dict(
          lat=(['nobs', 'nrows'], F['latitude'].T),
          lon=(['nobs', 'nrows'], F['longitude'].T),
          time=(time[0])
      ),
      attrs = dict(description="TROPOMI"),
    )
    data['NO2'].attrs.update({"cloud_threshold": 0.30,
                              "units": 'mol m-2'})
    data['NO2_std'][:] = 15e-6
    data['NO2_std'].attrs.update({"cloud_threshold": 0.30,
                                  "units": 'mol m-2'})

    data['NO2'].values[F['qa_value'].T < 0.75] = np.nan
    data['NO2_std'].values[F['qa_value'].T < 0.75] = np.nan

    return data